package de.hdm_stuttgart.mi.sd1.task2;

import java.util.Arrays;

/**
 * <p>Dealing with parcel shipping costs.</p>
 *
 * <p>Prices for rectangular parcels are depend on:</p>
 *
 * <ul>
 *     <li>Size dimensions length, width, height</li>
 *     <li>Weight</li>
 * </ul>
 *
 * <p><img src="doc-files/parcel.svg" alt="A parcels dimensions and weight" width="400" ></p>
 *
 * <p>Depending on these parameters different prices apply. There are 5 categories:</p>
 *
 * <ul>
 *     <li>{@link #PARCEL_SMALL_2KG}</li>
 *     <li>{@link #PARCEL_MEDIUM_2KG}</li>
 *     <li>{@link #PARCEL_5KG}</li>
 *     <li>{@link #PARCEL_10KG}</li>
 *     <li>{@link #PARCEL_31_5KG}</li>
 * </ul>
 */
public enum Parcel {

  /**
   * Parcel with {@code length <= 35 cm, width 25 <= cm, height 10 <= cm and weight 2000 <= g }
   */
  PARCEL_SMALL_2KG  ( 35, 25, 10,  2000,  379),
  /**
   * Parcel with {@code length <= 60cm, width <= 30cm, height <= 15cm and weight <= 2000g }
   */
  PARCEL_MEDIUM_2KG ( 60, 30, 15,  2000,  439),
  /**
   * Parcel with {@code length <= 120cm, width <= 60cm, height <= 60cm and weight <= 5000g }
   */
  PARCEL_5KG        (120, 60, 60,  5000,  599),
  /**
   * Parcel with {@code length <= 120cm, width <= 60cm, height <= 60cm and weight <= 10000g }
   */
  PARCEL_10KG       (120, 60, 60, 10000,  849),
  /**
   * Parcel with {@code length <= 120cm, width <= 60cm, height <= 60cm and weight <= 31500g }
   */
  PARCEL_31_5KG     (120, 60, 60, 31500, 1649);


  /**
   * A parcel's maximum limits for a given shipping cost category.
   *
   * Precondition: height <= width <= length
   *
   * @param length Parcel's length in centimeter
   * @param width Parcel's width in centimeter
   * @param height Parcel's height in centimeter
   * @param weight Parcel's weight in gramms (g)
   * @param shippingCost Parcel's shipping costs in Euro cents
   */
  Parcel(final int length, final int width, final int height, final int weight, final int shippingCost) {
    // TODO: Implement me correctly
  }

  /**
   * <p>Calculating the desired parcel's category</p>
   *
   * <p>The dimensions may be provided in <em>arbitrary</em> order.</p>
   *
   * <p>Example: A customer wants to ship a parcel of e.g. 34xm x 10cm x 40cm (given in that ordering) and 500g of weight. The
   * cheapest parcel category accommodating both sizes and weight is {@link #PARCEL_5KG}.</p>
   *
   * <section class="implementationHints">
   *     <h4>Hints:</h4>
   *
   *     <ul>
   *         <li>
   *             <p>Sorting <code>int</code> arrays using method {@link Arrays#sort(int[])}:</p>
   *             <pre><code class="java"> final int[] values = {1, 5, 3};
   * Arrays.sort(values);</code></pre>
   *         </li>
   *     </ul>
   * </section>
   *
   * @param length1 One of three parcel dimension sizes (length, width or height) in cm
   * @param length2 One of three parcel dimension sizes (length, width or height) in cm
   * @param length3 One of three parcel dimension sizes (length, width or height) in cm
   * @param weight  The parcel's weight in gramms.
   * @return The cheapest matching parcel category or <code>null</code> if no such exists.
   */
  static public Parcel cheapestCategory(final int length1, final int length2, final int length3, final int weight) {
    return PARCEL_5KG; // TODO: Implement me correctly
  }
}