package de.hdm_stuttgart.mi.sd1.task2;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import org.junit.Assert;
import org.junit.Test;

import static de.hdm_stuttgart.mi.sd1.task2.Parcel.*;

/**
 * Testing parcel shipping costs
 *
 */

public class ParcelTest extends ExaminationTestDefaults {

  static public final String DEBUG_TEST_HINT =
          "\n\nTest error: Wanna set a breakpoint having a debugging run?";


  public ParcelTest() {
    super(20000);
  }

  @Marking(points = 4)
  @Test
  public void testZuGross() {
    Assert.assertNull(cheapestCategory(121, 1,1, 10));
    Assert.assertNull(cheapestCategory(1, 433,1, 10));
    Assert.assertNull(cheapestCategory(20, 1,220, 10));
  }

  @Marking(points = 4)
  @Test
  public void testZuSchwer() {
    Assert.assertNull(cheapestCategory(12, 1,1, 60000));
    Assert.assertNull(cheapestCategory(1, 4,1, 31501));
    Assert.assertNull(cheapestCategory(20, 1,20, 40000));
  }

  @Marking(points = 6)
  @Test
  public void testWeight() {

    for (int length = 120; 0 < length; length -= 3) {
      for (int width = 60; 0 < width; width -= 2) {
        for (int height = 60; 0 < height; height -= 2) {

          for (int weight = 5000; 2000 < weight ; weight -= 231) {
            permuteDimensions(PARCEL_5KG, length, width, height, weight);
          }
          for (int weight = 10000; 5000 < weight; weight -= 317) {
            permuteDimensions(PARCEL_10KG, length, width, height, weight);
          }
          for (int weight = 31500; 10000 < weight ; weight -= 413) {
            permuteDimensions(PARCEL_31_5KG, length, width, height, weight);
          }
        }
      }
    }
  }


  @Marking(points = 6)
  @Test
  public void testSize() {
    for (int length = 35; 0 < length; length -= 2) {
      for (int width = 25; 0 < width; width--) {
        for (int height = 10; 0 < height; height--) {
          for (int weight = 2000; 0 < weight ; weight -= 31) {
            permuteDimensions(PARCEL_SMALL_2KG, length, width, height, weight);
          }
        }
      }
    }
    for (int length = 60; 35 < length; length -= 2) {
      for (int width = 30; 0 < width; width--) {
        for (int height = 15; 0 < height; height--) {
          for (int weight = 2000; 0 < weight ; weight -= 31) {
            permuteDimensions(PARCEL_MEDIUM_2KG, length, width, height, weight);
          }
        }
      }
    }
    for (int length = 120; 60 < length; length -= 5) {
      for (int width = 60; 0 < width; width -= 2) {
        for (int height = 60; 0 < height; height--) {
          for (int weight = 5000; 0 < weight ; weight -= 29) {
            permuteDimensions(PARCEL_5KG, length, width, height, weight);
          }
        }
      }
    }
  }

  private void permuteDimensions(final Parcel expectedCategory,
                                 final int length1, final int length2, final int length3, final int weight) {

    assertCategory(expectedCategory, length1, length2, length3, weight);
    assertCategory(expectedCategory, length1, length3, length2, weight);
    assertCategory(expectedCategory, length2, length1, length3, weight);
    assertCategory(expectedCategory, length2, length3, length1, weight);
    assertCategory(expectedCategory, length3, length1, length2, weight);
    assertCategory(expectedCategory, length3, length2, length1, weight);
  }

  private void assertCategory(final Parcel expectedCategory,
                              final int length1, final int length2, final int length3, final int weight) {
    try {
      Assert.assertEquals(expectedCategory, cheapestCategory(length1, length2, length3, weight));
    } catch (final AssertionError e) {
      final String message = "Expected " + expectedCategory + " for (" + length1 + "cm x " + length2 + "cm x " +
              length3 + "cm) and weight " + weight + "g";

      System.out.println(DEBUG_TEST_HINT);
      Assert.assertEquals(message, expectedCategory,
              cheapestCategory(length1, length2, length3, weight));
    }
  }
}